/**
 * Materialien zu den zentralen NRW-Abiturpruefungen im Fach Informatik ab 2017.
 * 
 * Generische Klasse BinarySearchTree<ContentType>
 * 
 * Mithilfe der generischen Klasse BinarySearchTree koennen beliebig viele
 * Objekte in einem Binaerbaum (binaerer Suchbaum) entsprechend einer
 * Ordnungsrelation verwaltet werden.
 * Ein Objekt der Klasse stellt entweder einen leeren binaeren Suchbaum dar oder
 * verwaltet ein Inhaltsobjekt sowie einen linken und einen rechten Teilbaum,
 * die ebenfalls Objekte der Klasse BinarySearchTree sind.
 * Die Klasse der Objekte, die in dem Suchbaum verwaltet werden sollen, muss
 * das generische Interface ComparableContent implementieren. Dabei muss durch
 * Ueberschreiben der drei Vergleichsmethoden isLess, isEqual, isGreater (s.
 * Dokumentation des Interfaces) eine eindeutige Ordnungsrelation festgelegt
 * sein.
 * Alle Objekte im linken Teilbaum sind kleiner als das Inhaltsobjekt des
 * binaeren Suchbaums. Alle Objekte im rechten Teilbaum sind groesser als das
 * Inhaltsobjekt des binaeren Suchbaums. Diese Bedingung gilt (rekursiv) auch in
 * beiden Teilbaeumen. 
 * Hinweis: In dieser Version wird die Klasse BinaryTree nicht benutzt.
 * 
 * @author Qualitaets- und UnterstuetzungsAgentur - Landesinstitut fuer Schule, Materialien zum schulinternen Lehrplan Informatik SII
 * @version Generisch_02 2014-02-21
 */
public class BinarySearchTree<ContentType extends ComparableContent<ContentType>> {

    /* --------- Anfang der privaten inneren Klasse -------------- */

    /**
     * Durch diese innere Klasse kann man dafuer sorgen, dass ein leerer Baum
     * null ist, ein nicht-leerer Baum jedoch immer eine nicht-null-Wurzel sowie
     * nicht-null-Teilbaeume hat.
     */
    private class BSTNode<CT extends ComparableContent<CT>> {

        private CT content;
        private BinarySearchTree<CT> left, right;

        public BSTNode(CT pContent) {
           // ToDo
        }

    }

    /* ----------- Ende der privaten inneren Klasse -------------- */

    private BSTNode<ContentType> node;

    /**
     * Der Konstruktor erzeugt einen leeren Suchbaum.
     */
    public BinarySearchTree() {
        // ToDo
    }

    /**
     * Diese Anfrage liefert den Wahrheitswert true, wenn der Suchbaum leer ist,
     * sonst liefert sie den Wert false.
     * 
     * @return true, wenn der binaere Suchbaum leer ist, sonst false
     * 
     */
    public boolean isEmpty() {
        return true; // Dummy
    }

    /**
     * Diese Anfrage liefert das Inhaltsobjekt des Suchbaumes. Wenn der Suchbaum
     * leer ist, wird null zurueckgegeben.
     * 
     * @return das Inhaltsobjekt vom Typ ContentType bzw. null, wenn der aktuelle
     *         Suchbaum leer ist
     *         
     */
    public ContentType getContent() {
        return null; // Dummy
    }

    /**
     * Diese Anfrage liefert den linken Teilbaum des binaeren Suchbaumes. 
     * Wenn er leer ist, wird null zurueckgegeben.
     * 
     * @return den linken Teilbaum (Objekt vom Typ BinarySearchTree<ContentType>) 
     *         bzw. null, wenn der Suchbaum leer ist
     *         
     */
    public BinarySearchTree<ContentType> getLeftTree() {
        return null; // Dummy
    }

    /**
     * Diese Anfrage liefert den rechten Teilbaum des binaeren Suchbaumes.
     * Wenn er leer ist, wird null zurueckgegeben.
     * 
     * @return den rechten Teilbaum (Objekt vom Typ BinarySearchTree<ContentType>) 
     *         bzw. null, wenn der aktuelle Suchbaum leer ist
     *         
     */
    public BinarySearchTree<ContentType> getRightTree() {
        return null; // Dummy
    }

    /**
     * Falls ein bezueglich der verwendeten Vergleichsmethode isEqual mit
     * pContent uebereinstimmendes Objekt im binaeren Suchbaum enthalten ist,
     * liefert die Anfrage dieses, ansonsten wird null zurueckgegeben.
     * Falls der Parameter null ist, wird null zurueckgegeben.
     * 
     * @param pContent
     *            zu suchendes Objekt vom Typ ContentType
     * @return das gefundene Objekt vom Typ ContentType, bei erfolgloser Suche null
     * 
     */
    public ContentType search(ContentType pContent) {
        return null; // Dummy
    }

    /**
     * Falls der Parameter null ist, geschieht nichts.
     * Falls ein bezueglich der verwendeten Vergleichsmethode isEqual mit
     * pContent uebereinstimmendes Objekt im geordneten binaeren Suchbaum
     * enthalten ist, passiert nichts. 
     * Achtung: hier wird davon ausgegangen, dass isEqual genau dann true
     * liefert, wenn isLess und isGreater false liefern.
     * Andernfalls (isLess oder isGreater) wird das Objekt pContent entsprechend
     * der vorgegebenen Ordnungsrelation in den BinarySearchTree eingeordnet.
     * 
     * @param pContent
     *            einzufuegendes Objekt vom Typ ContentType
     *            
     */
    public void insert(ContentType pContent) {
        // ToDo
    }

    /**
     * Falls ein bezueglich der verwendeten Vergleichsmethode mit
     * pContent uebereinstimmendes Objekt im binaeren Suchbaum enthalten
     * ist, wird dieses entfernt. Falls der Parameter null ist, aendert sich
     * nichts.
     * 
     * @param pContent
     *            zu entfernendes Objekt vom Typ ContentType
     *            
     */
    public void remove(ContentType pContent) {
        // ToDo
    }

    /* ----------- Weitere private Methoden -------------- 
    ----------- (sind hilfreich fr das Lschen) ------ */

    /**
     * Die Methode liefert denjenigen Baum, dessen linker Nachfolger keinen linken
     * Nachfolger mehr hat. Es ist also spaeter moeglich, in einem Baum im
     * rechten Nachfolger den Vorgaenger des linkesten Nachfolgers zu finden.
     * 
     */
    private BinarySearchTree<ContentType> ancestorOfSmallRight() {		
        return null; // Dummy
    }

    private BSTNode<ContentType> getNodeOfLeftSuccessor() {
        return node.left.node;
    }

    private BSTNode<ContentType> getNodeOfRightSuccessor() {
        return node.right.node;
    }
}

